#!/usr/bin/fades --system-site-packages

import datetime
import sys
from decimal import Decimal

import pyoo  # fades

CONNECT_MSG = """
Need to start Libreffice! Use a command like:
    libreoffice -accept="socket,host=localhost,port=2002;urp;"
"""

# all useful rows should be less than this
MAX_ROW_DATA = 3000

# epoch
BASE_DATE = datetime.date(1899, 12, 30)


class Record:
    """A typical record with expense info."""
    def __init__(self, triplet):
        date_int, self.description, amount = triplet
        self.date = datetime.timedelta(days=date_int) + BASE_DATE

        if not amount.startswith("="):
            amount = str(round(Decimal(amount), 2))
        self.amount = amount

    def for_paste(self):
        return "{:%d/%m/%Y}\t{}\t{:>15s}".format(self.date, self.description, self.amount)

    def __str__(self):
        return "[{:%Y-%m-%d}] {}: {}".format(self.date, self.description, self.amount)


def get_records(sheet, init_row, init_col):
    """Get the records from a pack."""
    def _producer():
        dates = sheet[init_row:MAX_ROW_DATA, init_col].values
        descrips = sheet[init_row:MAX_ROW_DATA, init_col + 1].values
        amounts = sheet[init_row:MAX_ROW_DATA, init_col + 2].formulas
        for item in zip(dates, descrips, amounts):
            yield item

    pack = _producer()

    # consume the header
    triplet = next(pack)
    assert triplet[1] != ''  # the title
    triplet = next(pack)
    assert triplet[0] == ''
    assert triplet[1] == ''
    assert triplet[2] != ''  # the total of the column
    triplet = next(pack)
    assert triplet == ('', '', '') or triplet == (',', '', ''), triplet  # the separator

    # read initial chunk (the visible one where we normally operate)
    initial_chunk = []
    for triplet in pack:
        if triplet[0] == '':
            # end of first chunk
            break
        initial_chunk.append(triplet)

    # consume the middle intermediate space
    for triplet in pack:
        if triplet[0] != '':
            break

    # grab the historic data
    historic = [triplet]  # include the one not empty that made last loop break
    for triplet in pack:
        if triplet[0] == '':
            break
        historic.append(triplet)
    else:
        raise ValueError("End of data achieved without emptiness, MAX_ROW_DATA not enough")

    # join and process
    all_data = [Record(triplet) for triplet in historic + initial_chunk]
    return all_data


def process_pack(sheet, init_row, init_col):
    """Process a pack of three columns."""
    records = get_records(sheet, init_row, init_col)

    # validate date ordering
    prev = records[0]
    for record in records[1:]:
        if record.date < prev.date:
            print("WARNING: bad date order!", prev, record)
        prev = record

    return records


cuit = sys.argv[1].replace("-", "").strip()

try:
    desktop = pyoo.Desktop('localhost', 2002)
except IOError:
    print(CONNECT_MSG)
    exit()
doc = desktop.open_spreadsheet("/home/facundo/Dropbox/Facundo/Distrib.ods")


COLLECT_INFO = {
    "Entrada": "Entrada",
    "Impuestos": "Impuestos",
    "Gastos generales": "Gastos",
}

sheet = doc.sheets["Trabajo"]
cells = sheet[12, 0:100].values  # row 12, columns 0:100
data = {}
for column, cell in enumerate(cells):
    if cell == "Trabajo Moni":
        pivot = column - 1
        results = process_pack(sheet, 12, pivot)
        break
else:
    print("ERROR: 'Trabajo Moni' not found")
    exit(1)

for result in results:
    if cuit in result.description:
        print(result.for_paste())
